@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -43,6 +44,10 @@
 #include <sc_options.h>
 #include <sc_statistics.h>
 
+/** We had 1. / 0. here to create a NaN but that is not portable. */
+#define TSEARCH_INVALID (-1.)
+static const double tsearch_invalid = TSEARCH_INVALID;
+
 static int          refine_level, level_shift;
 
 typedef enum tsearch_stats
@@ -402,9 +407,9 @@ tsearch_setup (tsearch_global_t * tsg)
   int                 retval;
   double              nref[P4EST_DIM];
 #endif
-  double              bbox[3][2] = { {1. / 0., -1. / 0.},
-  {1. / 0., -1. / 0.},
-  {1. / 0., -1. / 0.}
+  double              bbox[3][2] = { {TSEARCH_INVALID, TSEARCH_INVALID},
+  {TSEARCH_INVALID, TSEARCH_INVALID},
+  {TSEARCH_INVALID, TSEARCH_INVALID}
   };
 
   mlen = 1. / P4EST_ROOT_LEN;
@@ -449,7 +454,7 @@ tsearch_setup (tsearch_global_t * tsg)
   }
   else {
     tsg->R2 = 0.;
-    tsg->r2 = 1. / 0.;
+    tsg->r2 = tsearch_invalid;
     for (k = 0; k < P4EST_CHILDREN; ++k) {
       p4est_quadrant_corner_node (q, k, &c);
       ref[0] = c.x * mlen - 1. + (tsg->which_tree & 1);
@@ -668,7 +673,7 @@ time_search_1 (tsearch_global_t * tsg, p4est_t * p4est, size_t znum_points,
   for (zz = 0; zz < znum_points; ++zz) {
     tsg->matches = 0;
     sc_array_init_view (&pview, tsg->points, zz, 1);
-    p4est_search (p4est, time_search_fn, time_search_fn, &pview);
+    p4est_search_local (p4est, 0, time_search_fn, time_search_fn, &pview);
     ll += (long long) tsg->matches;
   }
   sc_flops_shot (fi, &snapshot);
@@ -711,7 +716,7 @@ time_search_1 (tsearch_global_t * tsg, p4est_t * p4est, size_t znum_points,
     }
 
     sc_stats_compute (tsg->mpicomm, 1, &raystat);
-    sc_stats_print (p4est_package_id, SC_LP_STATISTICS, 1, &raystat, 1, 0);
+    sc_stats_print (p4est_package_id, SC_LP_ESSENTIAL, 1, &raystat, 1, 0);
   }
 }
 
@@ -742,7 +747,7 @@ time_search_N (tsearch_global_t * tsg, p4est_t * p4est, size_t znum_points,
 
   tsg->matches = 0;
   sc_flops_snap (fi, &snapshot);
-  p4est_search (p4est, time_search_fn, time_search_fn, tsg->points);
+  p4est_search_local (p4est, 0, time_search_fn, time_search_fn, tsg->points);
   sc_flops_shot (fi, &snapshot);
   sc_stats_set1 (&stats[TSEARCH_SEARCH_N], snapshot.iwtime, "Search_N");
   ll = (long long) tsg->matches;
@@ -786,7 +791,7 @@ time_search_N (tsearch_global_t * tsg, p4est_t * p4est, size_t znum_points,
     }
 
     sc_stats_compute (tsg->mpicomm, 1, &raystat);
-    sc_stats_print (p4est_package_id, SC_LP_STATISTICS, 1, &raystat, 1, 0);
+    sc_stats_print (p4est_package_id, SC_LP_ESSENTIAL, 1, &raystat, 1, 0);
   }
 }
 
@@ -1043,7 +1048,7 @@ main (int argc, char **argv)
 
   /* calculate and print timings */
   sc_stats_compute (tsg->mpicomm, TSEARCH_NUM_STATS, stats);
-  sc_stats_print (p4est_package_id, SC_LP_STATISTICS,
+  sc_stats_print (p4est_package_id, SC_LP_ESSENTIAL,
                   TSEARCH_NUM_STATS, stats, 1, 1);
 
   /* destroy the p4est and its connectivity structure */
