@@ -39,8 +39,6 @@ typedef void        (*sc_sig_t) (int);
 #endif
 #endif
 
-#include <errno.h>
-
 #ifdef SC_ENABLE_PTHREAD
 #include <pthread.h>
 #endif
@@ -112,7 +110,6 @@ static sc_abort_handler_t sc_default_abort_handler = sc_abort_handler;
 static int          sc_signals_caught = 0;
 static sc_sig_t     system_int_handler = NULL;
 static sc_sig_t     system_segv_handler = NULL;
-static sc_sig_t     system_usr2_handler = NULL;
 
 static int          sc_print_backtrace = 0;
 
@@ -216,9 +213,6 @@ sc_signal_handler (int sig)
   case SIGSEGV:
     sigstr = "SEGV";
     break;
-  case SIGUSR2:
-    sigstr = "USR2";
-    break;
   default:
     sigstr = "<unknown>";
     break;
@@ -228,8 +222,8 @@ sc_signal_handler (int sig)
   sc_abort ();
 }
 
-/** Installs or removes a signal handler for INT SEGV USR2 that aborts.
- * \param [in] catch    If true, catch signals INT SEGV USR2.
+/** Installs or removes a signal handler for INT SEGV that aborts.
+ * \param [in] catch    If true, catch signals INT SEGV.
  *                      If false, reinstate previous signal handler.
  */
 static void
@@ -240,8 +234,6 @@ sc_set_signal_handler (int catch_signals)
     SC_CHECK_ABORT (system_int_handler != SIG_ERR, "catching INT");
     system_segv_handler = signal (SIGSEGV, sc_signal_handler);
     SC_CHECK_ABORT (system_segv_handler != SIG_ERR, "catching SEGV");
-    system_usr2_handler = signal (SIGUSR2, sc_signal_handler);
-    SC_CHECK_ABORT (system_usr2_handler != SIG_ERR, "catching USR2");
     sc_signals_caught = 1;
   }
   else if (!catch_signals && sc_signals_caught) {
@@ -249,8 +241,6 @@ sc_set_signal_handler (int catch_signals)
     system_int_handler = NULL;
     (void) signal (SIGSEGV, system_segv_handler);
     system_segv_handler = NULL;
-    (void) signal (SIGUSR2, system_usr2_handler);
-    system_usr2_handler = NULL;
     sc_signals_caught = 0;
   }
 }
@@ -367,8 +357,8 @@ sc_malloc_aligned (size_t alignment, size_t size)
      * size up front, then the real data shifted by at most alignment - 1
      * bytes.  This way there is always at least one stop byte at the end that
      * we can use for debugging. */
-    const ptrdiff_t     extrasize = (const ptrdiff_t) (2 * sizeof (char **));
-    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+    const ptrdiff_t     extrasize = (ptrdiff_t) (2 * sizeof (char **));
+    const ptrdiff_t     signalign = (ptrdiff_t) alignment;
     const size_t        alloc_size = extrasize + size + alignment;
     char               *alloc_ptr = (char *) malloc (alloc_size);
     char               *ptr;
@@ -427,8 +417,8 @@ sc_free_aligned (void *ptr, size_t alignment)
     /* this mirrors the function sc_malloc_aligned above */
     char               *alloc_ptr;
 #ifdef SC_ENABLE_DEBUG
-    const ptrdiff_t     extrasize = (const ptrdiff_t) (2 * sizeof (char **));
-    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+    const ptrdiff_t     extrasize = (ptrdiff_t) (2 * sizeof (char **));
+    const ptrdiff_t     signalign = (ptrdiff_t) alignment;
     ptrdiff_t           shift, modu, ssize, i;
 #endif
 
@@ -482,7 +472,7 @@ sc_realloc_aligned (void *ptr, size_t alignment, size_t size)
 #else
   {
 #ifdef SC_ENABLE_DEBUG
-    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+    const ptrdiff_t     signalign = (ptrdiff_t) alignment;
 #endif
     size_t              old_size, min_size;
     void               *new_ptr;
@@ -764,6 +754,19 @@ sc_double_compare (const void *v1, const void *v2)
   return d1 < d2 ? -1 : d1 > d2 ? 1 : 0;
 }
 
+int
+sc_atoi (const char *nptr)
+{
+  long                r = strtol (nptr, NULL, 10);
+  return r <= INT_MIN ? INT_MIN : r >= INT_MAX ? INT_MAX : (int) r;
+}
+
+long
+sc_atol (const char *nptr)
+{
+  return strtol (nptr, NULL, 10);
+}
+
 void
 sc_set_log_defaults (FILE * log_stream,
                      sc_log_handler_t log_handler, int log_threshold)
@@ -1428,4 +1431,63 @@ SC_LOG_IMP (ESSENTIAL, ESSENTIAL)
 SC_LOG_IMP (LERROR, ERROR)
 /* *INDENT-ON* */
 
+void
+sc_strcopy (char *dest, size_t size, const char *src)
+{
+  sc_snprintf (dest, size, "%s", src);
+}
+
+void
+sc_snprintf (char *str, size_t size, const char *fmt, ...)
+{
+  int                 retval;
+  va_list             ap;
+
+  /* If there is no space, we do not access the buffer at all.
+     Further down we expect it to be at least 1 byte wide */
+  if (str == NULL || size == 0) {
+    return;
+  }
+
+  /* Writing this function just to catch the return value.
+     Avoiding -Wnoformat-truncation gcc option this way */
+  va_start (ap, fmt);
+  retval = vsnprintf (str, size, fmt, ap);
+  if (retval < 0) {
+    str[0] = '\0';
+  }
+  /* We do not handle truncation, since it is expected in our design. */
+  va_end (ap);
+}
+
+const char         *
+sc_version (void)
+{
+  return SC_VERSION;
+}
+
+int
+sc_version_major (void)
+{
+  /* In rare cases SC_VERSION_MINOR may be a non-numerical string */
+  return sc_atoi (SC_TOSTRING (SC_VERSION_MAJOR));
+}
+
+int
+sc_version_minor (void)
+{
+  /* In rare cases SC_VERSION_MINOR may be a non-numerical string */
+  return sc_atoi (SC_TOSTRING (SC_VERSION_MINOR));
+}
+
+#if 0
+int
+sc_version_point (void)
+{
+  /* SC_VERSION_POINT may contain a dot and/or dash,
+     followed by additional information */
+  return sc_atoi (SC_TOSTRING (SC_VERSION_POINT));
+}
+#endif
+
 #endif
