@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -31,7 +32,7 @@ const char          sc_cmachchar[] =
 #ifndef SC_WITH_BLAS
 
 int
-sc_blas_nonimplemented ()
+sc_blas_nonimplemented (SC_NOARGS)
 {
   SC_ABORT ("BLAS not compiled in this configuration");
   return 0;
