@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -93,7 +94,7 @@ sc_dmatrix_new_internal (sc_bint_t m, sc_bint_t n, int init_zero)
   sc_dmatrix_t       *rdm;
   double             *data;
   size_t              size = (size_t) (m * n);
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   double              zero = 0.0;       /* no const to avoid warning */
   const double        anan = 0.0 / zero;
   size_t              zz;
@@ -108,7 +109,7 @@ sc_dmatrix_new_internal (sc_bint_t m, sc_bint_t n, int init_zero)
   }
   else {
     data = SC_ALLOC (double, size);
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
     /* In debug mode initialize the memory to NaN. */
     for (zz = 0; zz < size; ++zz) {
       data[zz] = anan;
@@ -170,6 +171,59 @@ sc_dmatrix_new_view_offset (sc_bint_t o, sc_bint_t m, sc_bint_t n,
   return rdm;
 }
 
+sc_dmatrix_t       *
+sc_dmatrix_new_view_column (sc_dmatrix_t * orig, sc_bint_t j)
+{
+  sc_dmatrix_t       *rdm;
+
+  SC_ASSERT (orig->m >= 0);
+  SC_ASSERT (0 <= j && j < orig->n);
+
+  rdm = SC_ALLOC (sc_dmatrix_t, 1);
+  sc_dmatrix_new_e (rdm, orig->m, orig->n, orig->e[0] + j);
+  rdm->n = 1;
+  rdm->view = 1;
+
+  return rdm;
+}
+
+void
+sc_dmatrix_view_set_column (sc_dmatrix_t * view,
+                            sc_dmatrix_t * orig, sc_bint_t j)
+{
+  const sc_bint_t     m = view->m;
+  sc_bint_t           i;
+
+  SC_ASSERT (view->view);
+  SC_ASSERT (view->m == orig->m);
+  SC_ASSERT (orig->m >= 0);
+  SC_ASSERT (0 <= j && j < orig->n);
+
+  view->e[0] = orig->e[0] + j;
+
+  if (m > 0) {
+    for (i = 1; i < m; ++i)
+      view->e[i] = view->e[i - 1] + orig->n;
+
+    view->e[m] = NULL;          /* safeguard */
+  }
+
+  view->n = 1;
+}
+
+void
+sc_dmatrix_view_set_row (sc_dmatrix_t * view,
+                         sc_dmatrix_t * orig, sc_bint_t i)
+{
+  SC_ASSERT (view->view);
+  SC_ASSERT (view->m == 1);
+  SC_ASSERT (orig->n >= 0);
+  SC_ASSERT (0 <= i && i < orig->m);
+
+  view->e[0] = orig->e[i];
+  view->n = orig->n;
+}
+
 sc_dmatrix_t       *
 sc_dmatrix_clone (const sc_dmatrix_t * X)
 {
@@ -212,7 +266,14 @@ sc_dmatrix_resize (sc_dmatrix_t * dmatrix, sc_bint_t m, sc_bint_t n)
   newsize = m * n;
 
   if (!dmatrix->view && size != newsize) {
+#ifdef SC_ENABLE_USE_REALLOC
     data = SC_REALLOC (dmatrix->e[0], double, newsize);
+#else
+    data = SC_ALLOC (double, newsize);
+    memcpy (data, dmatrix->e[0],
+            (size_t) SC_MIN (newsize, size) * sizeof (double));
+    SC_FREE (dmatrix->e[0]);
+#endif
   }
   else {
     /* for views you must know that data is large enough */
@@ -244,7 +305,14 @@ sc_dmatrix_resize_in_place (sc_dmatrix_t * dmatrix, sc_bint_t m, sc_bint_t n)
     }
   }
   if (newsize != size) {
+#ifdef SC_ENABLE_USE_REALLOC
     data = SC_REALLOC (dmatrix->e[0], double, newsize);
+#else
+    data = SC_ALLOC (double, newsize);
+    memcpy (data, dmatrix->e[0],
+            (size_t) SC_MIN (newsize, size) * sizeof (double));
+    SC_FREE (dmatrix->e[0]);
+#endif
   }
   if (n > old_n) {
     for (i = min_m - 1; i > 0; i--) {
@@ -333,6 +401,17 @@ sc_dmatrix_shift (double alpha, sc_dmatrix_t * X)
     Xdata[i] += alpha;
 }
 
+void
+sc_dmatrix_scale_shift (double alpha, double beta, sc_dmatrix_t * X)
+{
+  sc_bint_t           i;
+  const sc_bint_t     totalsize = X->m * X->n;
+  double             *Xdata = X->e[0];
+
+  for (i = 0; i < totalsize; ++i)
+    Xdata[i] = alpha * Xdata[i] + beta;
+}
+
 void
 sc_dmatrix_alphadivide (double alpha, sc_dmatrix_t * X)
 {
@@ -482,6 +561,24 @@ sc_dmatrix_dotdivide (const sc_dmatrix_t * X, sc_dmatrix_t * Y)
     Ydata[i] /= Xdata[i];
 }
 
+void
+sc_dmatrix_dotmultiply_add (const sc_dmatrix_t * A, const sc_dmatrix_t * X,
+                            sc_dmatrix_t * Y)
+{
+  sc_bint_t           i;
+  const sc_bint_t     totalsize = X->m * X->n;
+  const double       *Adata = A->e[0];
+  const double       *Xdata = X->e[0];
+  double             *Ydata = Y->e[0];
+
+  SC_ASSERT (X->m == A->m && X->n == A->n);
+  SC_ASSERT (X->m == Y->m && X->n == Y->n);
+
+  for (i = 0; i < totalsize; ++i) {
+    Ydata[i] += Adata[i] * Xdata[i];
+  }
+}
+
 void
 sc_dmatrix_copy (const sc_dmatrix_t * X, sc_dmatrix_t * Y)
 {
@@ -538,7 +635,7 @@ sc_dmatrix_vector (sc_trans_t transa, sc_trans_t transx, sc_trans_t transy,
 {
   sc_bint_t           inc = 1;
 
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   sc_bint_t           dimX = (transx == SC_NO_TRANS) ? X->m : X->n;
   sc_bint_t           dimY = (transy == SC_NO_TRANS) ? Y->m : Y->n;
   sc_bint_t           dimX1 = (transx == SC_NO_TRANS) ? X->n : X->m;
@@ -566,7 +663,7 @@ sc_dmatrix_multiply (sc_trans_t transa, sc_trans_t transb, double alpha,
                      double beta, sc_dmatrix_t * C)
 {
   sc_bint_t           Acols, Crows, Ccols;
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   sc_bint_t           Arows, Brows, Bcols;
 
   Arows = (transa == SC_NO_TRANS) ? A->m : A->n;
@@ -602,7 +699,7 @@ sc_dmatrix_ldivide (sc_trans_t transa, const sc_dmatrix_t * A,
   sc_trans_t          invtransa =
     (transa == SC_NO_TRANS) ? SC_TRANS : SC_NO_TRANS;
 
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   sc_bint_t           A_nrows = (transa == SC_NO_TRANS) ? A->m : A->n;
   sc_bint_t           A_ncols = (transa == SC_NO_TRANS) ? A->n : A->m;
   sc_bint_t           B_nrows = B->m;
@@ -631,7 +728,7 @@ sc_dmatrix_rdivide (sc_trans_t transb, const sc_dmatrix_t * A,
   sc_bint_t           A_nrows = A->m;
   sc_bint_t           B_nrows = (transb == SC_NO_TRANS) ? B->m : B->n;
   sc_bint_t           B_ncols = (transb == SC_NO_TRANS) ? B->n : B->m;
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   sc_bint_t           A_ncols = A->n;
   sc_bint_t           C_nrows = C->m;
   sc_bint_t           C_ncols = C->n;
@@ -649,14 +746,14 @@ sc_dmatrix_rdivide (sc_trans_t transb, const sc_dmatrix_t * A,
     /* Perform an LU factorization of B. */
     SC_LAPACK_DGETRF (&N, &N, lu->e[0], &N, ipiv, &info);
 
-    SC_ASSERT (info == 0);
+    SC_CHECK_ABORT (info == 0, "Lapack routine DGETRF failed");
 
     /* Solve the linear system. */
     sc_dmatrix_copy (A, C);
     SC_LAPACK_DGETRS (&sc_transchar[transb], &N, &Nrhs, lu->e[0], &N,
                       ipiv, C->e[0], &N, &info);
 
-    SC_ASSERT (info == 0);
+    SC_CHECK_ABORT (info == 0, "Lapack routine DGETRS failed");
 
     SC_FREE (ipiv);
     sc_dmatrix_destroy (lu);
@@ -666,6 +763,22 @@ sc_dmatrix_rdivide (sc_trans_t transb, const sc_dmatrix_t * A,
   }
 }
 
+void
+sc_dmatrix_solve_transpose_inplace (sc_dmatrix_t * A, sc_dmatrix_t * B)
+{
+  const sc_bint_t     N = A->m;
+  const sc_bint_t     nrhs = B->m;
+  sc_bint_t          *ipiv, info;
+
+  SC_ASSERT (A->n == N && B->n == N);
+
+  ipiv = SC_ALLOC (sc_bint_t, N);
+  SC_LAPACK_DGESV (&N, &nrhs, A->e[0], &N, ipiv, B->e[0], &N, &info);
+  SC_FREE (ipiv);
+
+  SC_CHECK_ABORT (info == 0, "Lapack routine DGESV failed");
+}
+
 void
 sc_dmatrix_write (const sc_dmatrix_t * dmatrix, FILE * fp)
 {
@@ -683,7 +796,7 @@ sc_dmatrix_write (const sc_dmatrix_t * dmatrix, FILE * fp)
 }
 
 sc_dmatrix_pool_t  *
-sc_dmatrix_pool_new (int m, int n)
+sc_dmatrix_pool_new (sc_bint_t m, sc_bint_t n)
 {
   sc_dmatrix_pool_t  *dmpool;
 
@@ -730,7 +843,7 @@ sc_dmatrix_pool_alloc (sc_dmatrix_pool_t * dmpool)
     dm = sc_dmatrix_new (dmpool->m, dmpool->n);
   }
 
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
   sc_dmatrix_set_value (dm, -1.);
 #endif
 
@@ -747,3 +860,54 @@ sc_dmatrix_pool_free (sc_dmatrix_pool_t * dmpool, sc_dmatrix_t * dm)
 
   *(sc_dmatrix_t **) sc_array_push (&dmpool->freed) = dm;
 }
+
+sc_darray_work_t   *
+sc_darray_work_new (const int n_threads, const int n_blocks,
+                    const int n_entries, const int alignment_bytes)
+{
+  const int           align_dbl = alignment_bytes / 8;
+  const int           n_entries_aligned = SC_ALIGN_UP (n_entries, align_dbl);
+  sc_darray_work_t   *work;
+
+  SC_ASSERT (0 < n_threads);
+  SC_ASSERT (0 < n_blocks);
+  SC_ASSERT (alignment_bytes <= 0 || (alignment_bytes % 8) == 0);
+
+  work = SC_ALLOC (sc_darray_work_t, 1);
+
+  work->data = SC_ALLOC (double, n_threads * n_blocks * n_entries_aligned);
+  work->n_threads = n_threads;
+  work->n_blocks = n_blocks;
+  work->n_entries = n_entries_aligned;
+
+  return work;
+}
+
+void
+sc_darray_work_destroy (sc_darray_work_t * work)
+{
+  SC_FREE (work->data);
+  SC_FREE (work);
+}
+
+double             *
+sc_darray_work_get (sc_darray_work_t * work, const int thread,
+                    const int block)
+{
+  SC_ASSERT (0 <= thread && thread < work->n_threads);
+  SC_ASSERT (0 <= block && block < work->n_blocks);
+
+  return work->data + work->n_entries * (work->n_blocks * thread + block);
+}
+
+int
+sc_darray_work_get_blockcount (sc_darray_work_t * work)
+{
+  return work->n_blocks;
+}
+
+int
+sc_darray_work_get_blocksize (sc_darray_work_t * work)
+{
+  return work->n_entries;
+}
