@@ -240,7 +240,7 @@ int                 sc_io_source_read_mirror (sc_io_source_t * source,
                                               size_t * bytes_out);
 
 /** This function writes numeric binary data in VTK base64 encoding.
- * \param vtkfile        Stream openened for writing.
+ * \param vtkfile        Stream opened for writing.
  * \param numeric_data   A pointer to a numeric data array.
  * \param byte_length    The length of the data array in bytes.
  * \return               Returns 0 on success, -1 on file error.
@@ -249,7 +249,7 @@ int                 sc_vtk_write_binary (FILE * vtkfile, char *numeric_data,
                                          size_t byte_length);
 
 /** This function writes numeric binary data in VTK compressed format.
- * \param vtkfile        Stream openened for writing.
+ * \param vtkfile        Stream opened for writing.
  * \param numeric_data   A pointer to a numeric data array.
  * \param byte_length    The length of the data array in bytes.
  * \return               Returns 0 on success, -1 on file error.
@@ -280,8 +280,25 @@ void                sc_fwrite (const void *ptr, size_t size,
 void                sc_fread (void *ptr, size_t size,
                               size_t nmemb, FILE * file, const char *errmsg);
 
+/** Best effort to flush a file's data to disc and close it.
+ * \param [in,out] file         File open for writing.
+ */
+void                sc_fflush_fsync_fclose (FILE * file);
+
 #ifdef SC_ENABLE_MPIIO
 
+/** Read MPI file content into memory.
+ * \param [in,out] mpifile      MPI file object opened for reading.
+ * \param [in] ptr      Data array to read from disk.
+ * \param [in] zcount   Number of array members.
+ * \param [in] t        The MPI type for each array member.
+ * \param [in] errmsg   Error message passed to SC_CHECK_ABORT.
+ * \note                This function aborts on MPI file and count errors.
+ */
+void                sc_mpi_read (MPI_File mpifile, const void *ptr,
+                                 size_t zcount, sc_MPI_Datatype t,
+                                 const char *errmsg);
+
 /** Write memory content to an MPI file.
  * \param [in,out] mpifile      MPI file object opened for writing.
  * \param [in] ptr      Data array to write to disk.
