@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,6 +30,11 @@
 
 SC_EXTERN_C_BEGIN;
 
+#ifdef SC_HAVE_LUA5_3_LUA_H
+#include <lua5.3/lua.h>
+#include <lua5.3/lualib.h>
+#include <lua5.3/lauxlib.h>
+#else
 #ifdef SC_HAVE_LUA5_2_LUA_H
 #include <lua5.2/lua.h>
 #include <lua5.2/lualib.h>
@@ -46,6 +52,7 @@ SC_EXTERN_C_BEGIN;
 #endif
 #endif
 #endif
+#endif
 
 SC_EXTERN_C_END;
 
