@@ -378,6 +378,13 @@ sc_MPI_Reduce (void *p, void *q, int n, sc_MPI_Datatype t,
   return sc_MPI_SUCCESS;
 }
 
+int
+sc_MPI_Reduce_scatter_block (void *p, void *q, int n, sc_MPI_Datatype t,
+                             sc_MPI_Op op, sc_MPI_Comm comm)
+{
+  return sc_MPI_Reduce (p, q, n, t, op, 0, comm);
+}
+
 int
 sc_MPI_Allreduce (void *p, void *q, int n, sc_MPI_Datatype t,
                   sc_MPI_Op op, sc_MPI_Comm comm)
@@ -463,35 +470,6 @@ sc_MPI_Wait (sc_MPI_Request * request, sc_MPI_Status * status)
   return sc_MPI_SUCCESS;
 }
 
-int
-sc_MPI_Waitsome (int incount, sc_MPI_Request * array_of_requests,
-                 int *outcount, int *array_of_indices,
-                 sc_MPI_Status * array_of_statuses)
-{
-  int                 i;
-
-  for (i = 0; i < incount; ++i) {
-    SC_CHECK_ABORT (array_of_requests[i] == sc_MPI_REQUEST_NULL,
-                    "non-MPI MPI_Waitsome handles NULL requests only");
-  }
-  *outcount = 0;
-
-  return sc_MPI_SUCCESS;
-}
-
-int
-sc_MPI_Waitall (int count, sc_MPI_Request * array_of_requests,
-                sc_MPI_Status * array_of_statuses)
-{
-  int                 i;
-
-  for (i = 0; i < count; ++i) {
-    SC_CHECK_ABORT (array_of_requests[i] == sc_MPI_REQUEST_NULL,
-                    "non-MPI MPI_Waitall handles NULL requests only");
-  }
-  return sc_MPI_SUCCESS;
-}
-
 double
 sc_MPI_Wtime (void)
 {
@@ -521,6 +499,64 @@ sc_MPI_Init_thread (int *argc, char ***argv, int required, int *provided)
 #endif /* !SC_ENABLE_MPITHREAD */
 #endif /* SC_ENABLE_MPI */
 
+int
+sc_MPI_Waitsome (int incount, sc_MPI_Request * array_of_requests,
+                 int *outcount, int *array_of_indices,
+                 sc_MPI_Status * array_of_statuses)
+{
+#ifdef SC_ENABLE_MPI
+  /* we do this to avoid warnings when the prototype uses [] */
+  return MPI_Waitsome (incount, array_of_requests, outcount,
+                       array_of_indices, array_of_statuses);
+#else
+  int                 i;
+
+  for (i = 0; i < incount; ++i) {
+    SC_CHECK_ABORT (array_of_requests[i] == sc_MPI_REQUEST_NULL,
+                    "non-MPI MPI_Waitsome handles NULL requests only");
+  }
+  *outcount = 0;
+
+  return sc_MPI_SUCCESS;
+#endif
+}
+
+int
+sc_MPI_Waitall (int count, sc_MPI_Request * array_of_requests,
+                sc_MPI_Status * array_of_statuses)
+{
+#ifdef SC_ENABLE_MPI
+  /* we do this to avoid warnings when the prototype uses [] */
+  return MPI_Waitall (count, array_of_requests, array_of_statuses);
+#else
+  int                 i;
+
+  for (i = 0; i < count; ++i) {
+    SC_CHECK_ABORT (array_of_requests[i] == sc_MPI_REQUEST_NULL,
+                    "non-MPI MPI_Waitall handles NULL requests only");
+  }
+  return sc_MPI_SUCCESS;
+#endif
+}
+
+int
+sc_MPI_Testall (int count, sc_MPI_Request * array_of_requests, int *flag,
+                sc_MPI_Status * array_of_statuses)
+{
+#ifdef SC_ENABLE_MPI
+  /* we do this to avoid warnings when the prototype uses [] */
+  return MPI_Testall (count, array_of_requests, flag, array_of_statuses);
+#else
+  int                 i;
+
+  for (i = 0; i < count; ++i) {
+    SC_CHECK_ABORT (array_of_requests[i] == sc_MPI_REQUEST_NULL,
+                    "non-MPI MPI_Testall handles NULL requests only");
+  }
+  return sc_MPI_SUCCESS;
+#endif
+}
+
 size_t
 sc_mpi_sizeof (sc_MPI_Datatype t)
 {
@@ -544,6 +580,8 @@ sc_mpi_sizeof (sc_MPI_Datatype t)
     return sizeof (double);
   if (t == sc_MPI_LONG_DOUBLE)
     return sizeof (long double);
+  if (t == sc_MPI_2INT)
+    return 2 * sizeof (int);
 
   SC_ABORT_NOT_REACHED ();
 }
