@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -102,7 +103,7 @@ sc_reduce_alltoall (sc_MPI_Comm mpicomm,
     /* process received data in the same order as sc_reduce_recursive */
     for (shift = 0, l = level - 1; l >= 0; ++shift, --l) {
       for (i = 0; i < 1 << l; ++i) {
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
         peer = sc_search_bias (maxlevel, l + 1, 2 * i, target);
 #endif
         peer2 = sc_search_bias (maxlevel, l + 1, 2 * i + 1, target);
