@@ -218,5 +218,5 @@ main (int argc, char **argv)
   mpiret = sc_MPI_Finalize ();
   SC_CHECK_MPI (mpiret);
 
-  return num_failed_tests ? 1 : 0;
+  return num_failed_tests ? EXIT_FAILURE : EXIT_SUCCESS;
 }
