@@ -52,15 +52,16 @@
 /*------------------------ QUERY API CHANGES -------------------------*/
 /*---- definitions to allow user code to query the p4est library -----*/
 
-/** We do no longer dereference unneeded pointers in p4est_transfer_.
- */
+/** We do no longer dereference unneeded pointers in p4est_transfer_. */
 #define P4EST_COMM_TRANSFER_NULL
 
 /** The \ref p4est_connectivity_new_disk function now accepts a bool arg.
- * The same holds for \ref p4est_wrap_new_disk.
- */
+ * The same holds for \ref p4est_wrap_new_disk. */
 #define P4EST_CONN_DISK_PERIODIC
 
+/** The \ref p4est_connectivity_reorder_newid function exists. */
+#define P4EST_CONN_REORDER_NEWID
+
 /** The \ref p4est_search_local function replaces \ref p4est_search.
  * The latter function is still available with updated internal semantics.
  * Furthermore, we have added \ref p4est_search_partition to search
@@ -69,6 +70,9 @@
  */
 #define P4EST_SEARCH_LOCAL
 
+/** We expose the \ref p4est_vtk_write_cell_datav function. */
+#define P4EST_VTK_CELL_DATAV
+
 /*--------------------------------------------------------------------*/
 
 SC_EXTERN_C_BEGIN;
@@ -76,6 +80,7 @@ SC_EXTERN_C_BEGIN;
 /** Typedef for quadrant coordinates. */
 typedef int32_t     p4est_qcoord_t;
 #define p4est_qcoord_compare sc_int32_compare
+#define P4EST_QCOORD_BITS 32
 #define P4EST_MPI_QCOORD sc_MPI_INT
 #define P4EST_VTK_QCOORD "Int32"
 #define P4EST_F90_QCOORD INTEGER(KIND=C_INT32_T)
@@ -87,6 +92,7 @@ typedef int32_t     p4est_qcoord_t;
 /** Typedef for counting topological entities (trees, tree vertices). */
 typedef int32_t     p4est_topidx_t;
 #define p4est_topidx_compare sc_int32_compare
+#define P4EST_TOPIDX_BITS 32
 #define P4EST_MPI_TOPIDX sc_MPI_INT
 #define P4EST_VTK_TOPIDX "Int32"
 #define P4EST_F90_TOPIDX INTEGER(KIND=C_INT32_T)
@@ -99,6 +105,7 @@ typedef int32_t     p4est_topidx_t;
 /** Typedef for processor-local indexing of quadrants and nodes. */
 typedef int32_t     p4est_locidx_t;
 #define p4est_locidx_compare sc_int32_compare
+#define P4EST_LOCIDX_BITS 32
 #define P4EST_MPI_LOCIDX sc_MPI_INT
 #define P4EST_VTK_LOCIDX "Int32"
 #define P4EST_F90_LOCIDX INTEGER(KIND=C_INT32_T)
@@ -110,6 +117,7 @@ typedef int32_t     p4est_locidx_t;
 /** Typedef for globally unique indexing of quadrants. */
 typedef int64_t     p4est_gloidx_t;
 #define p4est_gloidx_compare sc_int64_compare
+#define P4EST_GLOIDX_BITS 64
 #define P4EST_MPI_GLOIDX sc_MPI_LONG_LONG_INT
 #define P4EST_VTK_GLOIDX "Int64"
 #define P4EST_F90_GLOIDX INTEGER(KIND=C_INT64_T)
@@ -514,6 +522,28 @@ p4est_partition_cut_gloidx (p4est_gloidx_t global_num, int p, int num_procs)
   return result;
 }
 
+/** Return the full version of p4est.
+ *
+ * \return          Return the version of p4est using the format
+ *                  `VERSION_MAJOR.VERSION_MINOR.VERSION_POINT`,
+ *                  where `VERSION_POINT` can contain dots and
+ *                  characters, e.g. to indicate the additional
+ *                  number of commits and a git commit hash.
+ */
+const char         *p4est_version (void);
+
+/** Return the major version of p4est.
+ *
+ * \return          Return the major version of p4est.
+ */
+int                 p4est_version_major (void);
+
+/** Return the minor version of p4est.
+ *
+ * \return          Return the minor version of p4est.
+ */
+int                 p4est_version_minor (void);
+
 SC_EXTERN_C_END;
 
 #endif /* !P4EST_BASE_H */
