@@ -33,9 +33,15 @@
 #define P4EST_BITS_H
 
 #include <p4est.h>
+#include <sc_random.h>
 
 SC_EXTERN_C_BEGIN;
 
+/** Write -1 into the pad8 and pad16 members of a quadrant.
+ * This helps with valgrind cleanliness if a quadrant is sent over MPI.
+ */
+void                p4est_quadrant_pad (p4est_quadrant_t * q);
+
 /** Prints one line with quadrant's x, y and level.
  * \param [in] log_priority  see \ref logpriorities in sc.h for the meanings
  *                           of numerical priority values
@@ -50,6 +56,14 @@ void                p4est_quadrant_print (int log_priority,
 int                 p4est_quadrant_is_equal (const p4est_quadrant_t * q1,
                                              const p4est_quadrant_t * q2);
 
+/** Copy the Morton indices of the quadrant \a q.
+ *  \param[in] q      	 An extended quadrant.
+ *  \param[in,out] copy  An existing quadrant that Morton indices will
+ *                       be set to the Morton indices of \a q.
+ */
+void                p4est_quadrant_copy (const p4est_quadrant_t * q,
+                                         p4est_quadrant_t * copy);
+
 /** Test if two quadrants overlap.
  * \return true if \a q1 and \a q2 are equal or one is the ancestor of the
  * other.
@@ -338,7 +352,7 @@ void                p4est_quadrant_enlarge_last (const p4est_quadrant_t * a,
 /** Compute the ancestor of a quadrant at a given level.
  * \param [in]  q       Input quadrant.
  * \param [in]  level   A smaller level than q.
- * \param [in,out]  r   Existing quadrent whose Morton index will be filled
+ * \param [in,out]  r   Existing quadrant whose Morton index will be filled
  *                      with the ancestor of q at the given level.
  * \note The quadrant q may point to the same quadrant as r.
  *       The user_data of r are never modified.
@@ -654,6 +668,34 @@ uint64_t            p4est_quadrant_linear_id (const p4est_quadrant_t *
 void                p4est_quadrant_set_morton (p4est_quadrant_t * quadrant,
                                                int level, uint64_t id);
 
+/** Compute the successor according to the Morton index in a uniform mesh.
+ * \param[in] quadrant  Quadrant whose Morton successor will be computed.
+ *                      Must not be the last (top right) quadrant in the tree.
+ * \param[in,out] result    The coordinates and level of the successor of
+ *                          \b quadrant will be saved in \b result.
+ */
+void                p4est_quadrant_successor (const p4est_quadrant_t *
+                                              quadrant,
+                                              p4est_quadrant_t * result);
+
+/** Compute the predecessor according to the Morton index in a uniform mesh.
+ * \param[in] quadrant  Quadrant whose Morton predecessor will be computed.
+ *                      Must not be the first (bottom left) quadrant in the tree.
+ * \param[in,out] result    The coordinates and level of the predecessor of
+ *                          \b quadrant will be saved in \b result.
+ */
+void                p4est_quadrant_predecessor (const p4est_quadrant_t *
+                                                quadrant,
+                                                p4est_quadrant_t * result);
+
+/** Initialize a random number generator by quadrant coordinates.
+ * This serves to generate partition-independent and reproducible samples.
+ * \param [in]  q               Valid quadrant.
+ * \param [out] rstate          New state of random number generator.
+ */
+void                p4est_quadrant_srand (const p4est_quadrant_t * q,
+                                          sc_rand_state_t * rstate);
+
 SC_EXTERN_C_END;
 
 #endif /* !P4EST_BITS_H */
