@@ -958,10 +958,7 @@ p4est_transfer_assign_comm (const p4est_gloidx_t * dest_gfq,
                 src_gfq[*mpirank + 1] <= src_gfq[*mpisize]);
 }
 
-/** Given target, find index p such that gfq[p] <= target < gfq[p + 1].
- * \param [in] nmemb    Number of entries in array MINUS ONE.
- */
-static int
+int
 p4est_bsearch_partition (p4est_gloidx_t target,
                          const p4est_gloidx_t * gfq, int nmemb)
 {
@@ -1025,7 +1022,9 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
     /* our process as the receiver is not empty */
     first_sender = p4est_bsearch_partition (dest_begin, src_gfq, mpisize);
     P4EST_ASSERT (0 <= first_sender && first_sender < mpisize);
-    last_sender = p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    last_sender =
+      p4est_bsearch_partition (dest_end - 1, &src_gfq[first_sender],
+                               mpisize - first_sender) + first_sender;
     P4EST_ASSERT (first_sender <= last_sender && last_sender < mpisize);
     tc->num_senders = last_sender - first_sender + 1;
     P4EST_ASSERT (tc->num_senders > 0);
@@ -1080,7 +1079,9 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
     /* our process as the sender is not empty */
     first_receiver = p4est_bsearch_partition (src_begin, dest_gfq, mpisize);
     P4EST_ASSERT (0 <= first_receiver && first_receiver < mpisize);
-    last_receiver = p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    last_receiver =
+      p4est_bsearch_partition (src_end - 1, &dest_gfq[first_receiver],
+                               mpisize - first_receiver) + first_receiver;
     P4EST_ASSERT (first_receiver <= last_receiver && last_receiver < mpisize);
     tc->num_receivers = last_receiver - first_receiver + 1;
     P4EST_ASSERT (tc->num_receivers > 0);
@@ -1139,7 +1140,7 @@ p4est_transfer_fixed_begin (const p4est_gloidx_t * dest_gfq,
   return tc;
 }
 
-static void
+void
 p4est_transfer_end (p4est_transfer_context_t * tc)
 {
   int                 mpiret;
@@ -1168,7 +1169,7 @@ void
 p4est_transfer_fixed_end (p4est_transfer_context_t * tc)
 {
   P4EST_ASSERT (tc != NULL);
-  P4EST_ASSERT (!tc->variable);
+  P4EST_ASSERT (tc->variable == 0);
 
   p4est_transfer_end (tc);
 }
@@ -1185,21 +1186,25 @@ p4est_transfer_custom (const p4est_gloidx_t * dest_gfq,
   tc = p4est_transfer_custom_begin (dest_gfq, src_gfq, mpicomm, tag,
                                     dest_data, dest_sizes,
                                     src_data, src_sizes);
-  p4est_transfer_end (tc);
+  p4est_transfer_custom_end (tc);
 }
 
-p4est_transfer_context_t *
-p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
-                             const p4est_gloidx_t * src_gfq,
-                             sc_MPI_Comm mpicomm, int tag,
-                             void *dest_data, const int *dest_sizes,
-                             const void *src_data, const int *src_sizes)
+static p4est_transfer_context_t *
+p4est_transfer_begin (const p4est_gloidx_t * dest_gfq,
+                      const p4est_gloidx_t * src_gfq,
+                      sc_MPI_Comm mpicomm, int tag,
+                      void *dest_data, const int *dest_sizes,
+                      const void *src_data, const int *src_sizes,
+                      size_t item_size, int variable)
 {
   p4est_transfer_context_t *tc;
   int                 mpiret;
   int                 mpisize, mpirank;
   int                 q;
   int                 first_sender, last_sender;
+#ifdef P4EST_ENABLE_DEBUG
+  int                 old_last_sender, old_last_receiver;
+#endif
   int                 first_receiver, last_receiver;
   int                 i, ilen;
   const int          *rs;
@@ -1211,9 +1216,18 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
   p4est_gloidx_t      gbegin, gend;
   sc_MPI_Request     *rq;
 
+  /* consistency of internal helper function */
+  P4EST_ASSERT (variable == 1 || variable == 2);
+  P4EST_ASSERT ((variable == 1 && item_size == 1) || variable == 2);
+
   /* setup context structure */
   tc = P4EST_ALLOC_ZERO (p4est_transfer_context_t, 1);
-  tc->variable = 1;
+  tc->variable = variable;
+
+  /* there is nothing to do when there is no data */
+  if (item_size == 0) {
+    return tc;
+  }
 
   /* grab local partition information */
   p4est_transfer_assign_comm (dest_gfq, src_gfq, mpicomm, &mpisize, &mpirank);
@@ -1233,7 +1247,14 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
     /* our process as the receiver is not empty */
     first_sender = p4est_bsearch_partition (dest_begin, src_gfq, mpisize);
     P4EST_ASSERT (0 <= first_sender && first_sender < mpisize);
-    last_sender = p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    last_sender =
+      p4est_bsearch_partition (dest_end - 1, &src_gfq[first_sender],
+                               mpisize - first_sender) + first_sender;
+#ifdef P4EST_ENABLE_DEBUG
+    old_last_sender =
+      p4est_bsearch_partition (dest_end - 1, src_gfq, mpisize);
+    P4EST_ASSERT (last_sender == old_last_sender);
+#endif
     P4EST_ASSERT (first_sender <= last_sender && last_sender < mpisize);
     tc->num_senders = last_sender - first_sender + 1;
     P4EST_ASSERT (tc->num_senders > 0);
@@ -1258,7 +1279,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
       byte_len = 0;
       ilen = (int) (gend - gbegin);
       for (i = 0; i < ilen; ++i) {
-        byte_len += *rs++;
+        byte_len += item_size * *rs++;
       }
 
       /* choose how to treat the sender process */
@@ -1293,7 +1314,14 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
     /* our process as the sender is not empty */
     first_receiver = p4est_bsearch_partition (src_begin, dest_gfq, mpisize);
     P4EST_ASSERT (0 <= first_receiver && first_receiver < mpisize);
-    last_receiver = p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    last_receiver =
+      p4est_bsearch_partition (src_end - 1, &dest_gfq[first_receiver],
+                               mpisize - first_receiver) + first_receiver;
+#ifdef P4EST_ENABLE_DEBUG
+    old_last_receiver =
+      p4est_bsearch_partition (src_end - 1, dest_gfq, mpisize);
+    P4EST_ASSERT (last_receiver == old_last_receiver);
+#endif
     P4EST_ASSERT (first_receiver <= last_receiver && last_receiver < mpisize);
     tc->num_receivers = last_receiver - first_receiver + 1;
     P4EST_ASSERT (tc->num_receivers > 0);
@@ -1318,7 +1346,7 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
       byte_len = 0;
       ilen = (int) (gend - gbegin);
       for (i = 0; i < ilen; ++i) {
-        byte_len += *rs++;
+        byte_len += item_size * *rs++;
       }
 
       /* choose how to treat the receiver process */
@@ -1357,11 +1385,61 @@ p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
   return tc;
 }
 
+p4est_transfer_context_t *
+p4est_transfer_custom_begin (const p4est_gloidx_t * dest_gfq,
+                             const p4est_gloidx_t * src_gfq,
+                             sc_MPI_Comm mpicomm, int tag,
+                             void *dest_data, const int *dest_sizes,
+                             const void *src_data, const int *src_sizes)
+{
+  return p4est_transfer_begin
+    (dest_gfq, src_gfq, mpicomm, tag,
+     dest_data, dest_sizes, src_data, src_sizes, 1, 1);
+}
+
+p4est_transfer_context_t *
+p4est_transfer_items_begin (const p4est_gloidx_t * dest_gfq,
+                            const p4est_gloidx_t * src_gfq,
+                            sc_MPI_Comm mpicomm, int tag,
+                            void *dest_data, const int *dest_counts,
+                            const void *src_data, const int *src_counts,
+                            size_t item_size)
+{
+  return p4est_transfer_begin
+    (dest_gfq, src_gfq, mpicomm, tag,
+     dest_data, dest_counts, src_data, src_counts, item_size, 2);
+}
+
 void
 p4est_transfer_custom_end (p4est_transfer_context_t * tc)
 {
   P4EST_ASSERT (tc != NULL);
-  P4EST_ASSERT (tc->variable);
+  P4EST_ASSERT (tc->variable == 1);
+
+  p4est_transfer_end (tc);
+}
+
+void
+p4est_transfer_items (const p4est_gloidx_t * dest_gfq,
+                      const p4est_gloidx_t * src_gfq,
+                      sc_MPI_Comm mpicomm, int tag,
+                      void *dest_data, const int *dest_sizes,
+                      const void *src_data, const int *src_sizes,
+                      size_t item_size)
+{
+  p4est_transfer_context_t *tc;
+
+  tc = p4est_transfer_items_begin (dest_gfq, src_gfq, mpicomm, tag,
+                                   dest_data, dest_sizes,
+                                   src_data, src_sizes, item_size);
+  p4est_transfer_items_end (tc);
+}
+
+void
+p4est_transfer_items_end (p4est_transfer_context_t * tc)
+{
+  P4EST_ASSERT (tc != NULL);
+  P4EST_ASSERT (tc->variable == 2);
 
   p4est_transfer_end (tc);
 }
