@@ -54,6 +54,15 @@ SC_EXTERN_C_BEGIN;
 /** The size of insulation layer */
 #define P4EST_INSUL 9
 
+/** Only use logical AND term in 3D */
+#define P4EST_ONLY_P8_LAND(x)
+
+/** Only use comma and expression in 3D */
+#define P4EST_ONLY_P8_COMMA(x)
+
+/** Exponentiate with dimension */
+#define P4EST_DIM_POW(a) ((a) * (a))
+
 /* size of face transformation encoding */
 #define P4EST_FTRANSFORM 9
 
@@ -352,7 +361,7 @@ p4est_connectivity_t *p4est_connectivity_inflate (sc_array_t * buffer);
  * \return              Returns valid connectivity, or NULL on file error.
  */
 p4est_connectivity_t *p4est_connectivity_load (const char *filename,
-                                               size_t * bytes);
+                                               size_t *bytes);
 
 /** Create a connectivity structure for the unit square.
  */
@@ -401,6 +410,13 @@ p4est_connectivity_t *p4est_connectivity_new_star (void);
  */
 p4est_connectivity_t *p4est_connectivity_new_cubed (void);
 
+/** Create a connectivity structure for a five-tree flat spherical disk.
+ * This disk can just as well be used as a square to test non-Cartesian maps.
+ * Without any mapping this connectivity covers the square [-3, 3]**2.
+ * \return                      Initialized and usable connectivity.
+ */
+p4est_connectivity_t *p4est_connectivity_new_disk_nonperiodic (void);
+
 /** Create a connectivity structure for a five-tree flat spherical disk.
  * This disk can just as well be used as a square to test non-Cartesian maps.
  * Without any mapping this connectivity covers the square [-3, 3]**2.
@@ -418,12 +434,58 @@ p4est_connectivity_t *p4est_connectivity_new_cubed (void);
  * The general shape and periodicity are the same as those obtained with
  * \ref p4est_connectivity_new_brick (1, 1, periodic_a, periodic_b).
  *
+ * When setting \a periodic_a and \a periodic_b to false, the result is
+ * the same as that of \ref p4est_connectivity_new_disk_nonperiodic.
+ *
  * \param [in] periodic_a       Bool to make disk periodic in x direction.
  * \param [in] periodic_b       Bool to make disk periodic in y direction.
+ * \return                      Initialized and usable connectivity.
  */
 p4est_connectivity_t *p4est_connectivity_new_disk (int periodic_a,
                                                    int periodic_b);
 
+/**
+ * Create a connectivity for mapping the sphere using an icosahedron.
+ *
+ * The regular icosadron is a polyhedron with 20 faces, each of which is
+ * an equilateral triangle. To build the p4est connectivity, we group faces
+ * 2 by 2 to from 10 quadrangles, and thus 10 trees.
+ *
+ * This connectivity is meant to be used together with \ref p4est_geometry_new_icosahedron
+ * to map the sphere.
+ *
+ * The flat connectivity looks like that:
+ * Vextex numbering:
+ *
+ *    A00   A01   A02   A03   A04
+ *   /   \ /   \ /   \ /   \ /   \
+ * A05---A06---A07---A08---A09---A10
+ *   \   / \   / \   / \   / \   / \
+ *    A11---A12---A13---A14---A15---A16
+ *      \  /  \  /  \  /  \  /  \  /
+ *      A17   A18   A19   A20   A21
+ *
+ * Origin in A05.
+ *
+ * Tree numbering:
+ *
+ * 0  2  4  6  8
+ *  1  3  5  7  9
+ */
+p4est_connectivity_t *p4est_connectivity_new_icosahedron ();
+
+/** Create a connectivity structure that builds a 2d spherical shell.
+ * \ref p8est_connectivity_new_shell
+ */
+p4est_connectivity_t *p4est_connectivity_new_shell2d (void);
+
+/** Create a connectivity structure that maps a 2d disk.
+ *
+ * This is a 5 trees connectivity meant to be used together
+ * with \ref p4est_geometry_new_disk2d to map the disk.
+ */
+p4est_connectivity_t *p4est_connectivity_new_disk2d (void);
+
 /** A rectangular m by n array of trees with configurable periodicity.
  * The brick is periodic in x and y if periodic_a and periodic_b are true,
  * respectively.
@@ -452,8 +514,9 @@ p4est_connectivity_t *p4est_connectivity_new_byname (const char *name);
  * This is useful if you would like to uniformly refine by something other
  * than a power of 2.
  *
- * \param [in] conn         a valid connectivity
- * \param [in] num_per_edge the number of new trees in each direction
+ * \param [in] conn         A valid connectivity
+ * \param [in] num_per_edge The number of new trees in each direction.
+ *                      Must use no more than \ref P4EST_OLD_QMAXLEVEL bits.
  *
  * \return a refined connectivity.
  */
@@ -545,7 +608,7 @@ void                p4est_connectivity_permute (p4est_connectivity_t * conn,
                                                 int is_current_to_new);
 #ifdef P4EST_WITH_METIS
 
-/** p4est_connectivity_reorder
+/** Reorder a connectivity using METIS.
  * This function takes a connectivity \a conn and a parameter \a k,
  * which will typically be the number of processes, and reorders the trees
  * such that if every processes is assigned (num_trees / k) trees, the
@@ -571,6 +634,33 @@ void                p4est_connectivity_reorder (sc_MPI_Comm comm, int k,
                                                 p4est_connectivity_t * conn,
                                                 p4est_connect_type_t ctype);
 
+/** Reorder a connectivity using METIS.
+ * This is the same form as \ref p4est_connectivity_reorder but it takes an
+ * initialized sc array \a newid as extra argument.
+ * In this way, the users can map old indices to new indices in the case it
+ * is necessary (for instance to retrieve high-order nodes previously stored
+ * in an array with old indices).
+ * \param [in]     comm       MPI communicator.
+ * \param [in]     k          if k > 0, the number of pieces metis will use to
+ *                            guide the reordering; if k = 0, the number of
+ *                            pieces will be determined from the MPI
+ *                            communicator.
+ * \param [in,out] conn       connectivity that will be reordered.
+ * \param [in]     ctype      determines when an edge exists in the dual graph
+ *                            of the connectivity structure.
+ * \param [in,out] newid      array that maps old tree indices to new ones.
+ *                            newid has to be an sc_array and it has to be
+ *                            initialized (non-NULL) with element size
+ *                            of size_t (using sc_array_new (sizeof (size_t))).
+ *                            Input length arbitrary, output length modified.
+ */
+sc_array_t         *p4est_connectivity_reorder_newid (sc_MPI_Comm comm, int k,
+                                                      p4est_connectivity_t *
+                                                      conn,
+                                                      p4est_connect_type_t
+                                                      ctype,
+                                                      sc_array_t * newid);
+
 #endif /* P4EST_WITH_METIS */
 
 /** p4est_connectivity_join_faces
