@@ -81,6 +81,32 @@ void                p4est_geometry_destroy (p4est_geometry_t * geom);
 p4est_geometry_t   *p4est_geometry_new_connectivity (p4est_connectivity_t *
                                                      conn);
 
+/** Create a geometry for mapping the 3d sphere using 2d connectivity icosahedron.
+ *
+ */
+p4est_geometry_t   *p4est_geometry_new_icosahedron (p4est_connectivity_t *
+                                                    conn, double R);
+
+/** Create a geometry for mapping 2d shell.
+ *  This a direct adaptation of geometric shell in 3d.
+ */
+p4est_geometry_t   *p4est_geometry_new_shell2d (p4est_connectivity_t * conn,
+                                                double R2, double R1);
+
+/**
+ * disk2d geometry associated to disk2d connectivity.
+ *
+ * \param[in] R0 radius of the inner circle
+ * \param[in] R1 radius of the outer circle (external border)
+ *
+ *
+ * This geometry is meant to be used with the disk2d connectivity,
+ * \ref p4est_connectivity_new_disk2d which is a 5-tree connectivty
+ * to map the disk.
+ */
+p4est_geometry_t   *p4est_geometry_new_disk2d (p4est_connectivity_t * conn,
+                                               double R0, double R1);
+
 SC_EXTERN_C_END;
 
 #endif /* !P4EST_GEOMETRY_H */
