@@ -36,71 +36,89 @@
 
 SC_EXTERN_C_BEGIN;
 
-/** This structure contains complete mesh information on the forest.
+/** This structure contains complete mesh information on a 2:1 balanced forest.
  * It stores the locally relevant neighborhood, that is, all locally owned
  * quadrants and one layer of adjacent ghost quadrants and their owners.
  *
- * For each local quadrant, its tree number is stored in quad_to_tree. The
- * quad_to_tree array is NULL by default and can be enabled using
- * p4est_mesh_new_ext.
+ * For each local quadrant, its tree number is stored in quad_to_tree.
+ * The quad_to_tree array is NULL by default and can be enabled using
+ * \ref p4est_mesh_new_ext.
  * For each ghost quadrant, its owner rank is stored in ghost_to_proc.
  * For each level, an array of local quadrant numbers is stored in quad_level.
  * The quad_level array is NULL by default and can be enabled using
- * p4est_mesh_new_ext.
+ * \ref p4est_mesh_new_ext.
  *
  * The quad_to_quad list stores one value for each local quadrant's face.
  * This value is in 0..local_num_quadrants-1 for local quadrants, or in
  * local_num_quadrants + (0..ghost_num_quadrants-1) for ghost quadrants.
- * The quad_to_face list has equally many entries which are either:
+ *
+ * The quad_to_face list has equally many entries that are either:
  * 1. A value of v = 0..7 indicates one same-size neighbor.
  *    This value is decoded as v = r * 4 + nf, where nf = 0..3 is the
  *    neighbor's connecting face number and r = 0..1 is the relative
- *    orientation of the neighbor's face, see p4est_connectivity.h.
+ *    orientation of the neighbor's face; see p4est_connectivity.h.
  * 2. A value of v = 8..23 indicates a double-size neighbor.
  *    This value is decoded as v = 8 + h * 8 + r * 4 + nf, where
  *    r and nf are as above and h = 0..1 is the number of the subface.
+ *    h designates the subface of the large neighbor that the quadrant
+ *    touches (this is the same as the large neighbor's face corner).
  * 3. A value of v = -8..-1 indicates two half-size neighbors.
  *    In this case the corresponding quad_to_quad index points into the
- *    quad_to_half array which stores two quadrant numbers per index,
+ *    quad_to_half array that stores two quadrant numbers per index,
  *    and the orientation of the smaller faces follows from 8 + v.
  *    The entries of quad_to_half encode between local and ghost quadrant
  *    in the same way as the quad_to_quad values described above.
+ *    The small neighbors in quad_to_half are stored in the sequence
+ *    of the face corners of this, i.e., the large quadrant.
+ *
  * A quadrant on the boundary of the forest sees itself and its face number.
  *
  * The quad_to_corner list stores corner neighbors that are not face neighbors.
  * On the inside of a tree, there is precisely one such neighbor per corner.
  * In this case, its index is encoded as described above for quad_to_quad.
- * The neighbor's matching corner number is always diagonally opposite.
+ * The neighbor's matching corner number is always diagonally opposite,
+ * that is, corner number ^ 3.
  *
  * On the inside of an inter-tree face, we have precisely one corner neighbor.
  * If a corner is an inter-tree corner, then the number of corner neighbors
  * may be any non-negative number.  In both cases, the quad_to_corner value
  * is in
- *    local_num_quadrants + local_num_ghosts + [0 .. local_num_corners - 1]
- * where the offset by local quadrants and ghosts is implicitly substracted.
- * It indexes into corner_offset, which encodes a group of corner neighbors.
+ *    local_num_quadrants + local_num_ghosts + [0 .. local_num_corners - 1].
+ * After subtracting the number of local and ghost quadrants,
+ * it indexes into corner_offset, which encodes a group of corner neighbors.
  * Each group contains the quadrant numbers encoded as usual for quad_to_quad
  * in corner_quad, and the corner number from the neighbor as corner_corner.
  *
- * Corners with no diagonal neighbor at all are assigned the value -1.
+ * Corners with no diagonal neighbor at all are assigned the value -3.  This
+ * only happens on the domain boundary, which is necessarily a tree boundary.
+ * Corner-neighbors for hanging nodes are assigned the value -1.
+ *
+ * TODO: In case of an inter-tree corner neighbor relation in a brick-like
+ *       situation (exactly one neighbor, diagonally opposite corner number),
+ *       use the same encoding as for corners within a tree.
  */
 typedef struct
 {
   p4est_locidx_t      local_num_quadrants;
   p4est_locidx_t      ghost_num_quadrants;
 
-  p4est_topidx_t     *quad_to_tree;     /**< tree index for each local quad,
-                                             NULL by default */
+  p4est_topidx_t     *quad_to_tree;     /**< tree index for each local quad.
+                                               Is NULL by default, but may be
+                                             enabled by \ref p4est_mesh_new_ext. */
   int                *ghost_to_proc;    /**< processor for each ghost quad */
 
   p4est_locidx_t     *quad_to_quad;     /**< one index for each of the 4 faces */
   int8_t             *quad_to_face;     /**< encodes orientation/2:1 status */
   sc_array_t         *quad_to_half;     /**< stores half-size neighbors */
-  sc_array_t         *quad_level;       /**< stores lists of per-level quads,
-                                             NULL by default */
 
-  /* These members are NULL if the connect_t is not P4EST_CONNECT_CORNER */
-  /* CAUTION: tree-boundary corners not yet implemented */
+  sc_array_t         *quad_level;       /**< Stores lists of per-level quads.
+                                             The array has entries indexed by
+                                             0..P4EST_QMAXLEVEL inclusive that
+                                             are arrays of local quadrant ids.
+                                               Is NULL by default, but may be
+                                             enabled by \ref p4est_mesh_new_ext. */
+
+  /* These members are NULL if corners are not requested in \ref p4est_mesh_new. */
   p4est_locidx_t      local_num_corners;        /* tree-boundary corners */
   p4est_locidx_t     *quad_to_corner;   /* 4 indices for each local quad */
   sc_array_t         *corner_offset;    /* local_num_corners + 1 entries */
@@ -140,12 +158,15 @@ p4est_mesh_face_neighbor_t;
 size_t              p4est_mesh_memory_used (p4est_mesh_t * mesh);
 
 /** Create a p4est_mesh structure.
+ * This function does not populate the quad_to_tree and quad_level fields.
+ * To populate them, use \ref p4est_mesh_new_ext.
  * \param [in] p4est    A forest that is fully 2:1 balanced.
  * \param [in] ghost    The ghost layer created from the provided p4est.
  * \param [in] btype    Determines the highest codimension of neighbors.
  * \return              A fully allocated mesh structure.
  */
-p4est_mesh_t       *p4est_mesh_new (p4est_t * p4est, p4est_ghost_t * ghost,
+p4est_mesh_t       *p4est_mesh_new (p4est_t * p4est,
+                                    p4est_ghost_t * ghost,
                                     p4est_connect_type_t btype);
 
 /** Destroy a p4est_mesh structure.
@@ -153,9 +174,78 @@ p4est_mesh_t       *p4est_mesh_new (p4est_t * p4est, p4est_ghost_t * ghost,
  */
 void                p4est_mesh_destroy (p4est_mesh_t * mesh);
 
+/** Access a process-local quadrant inside a forest.
+ * Needs a mesh with populated quad_to_tree array.
+ * This is a special case of \ref p4est_mesh_quadrant_cumulative.
+ *
+ * \param [in] p4est  The forest.
+ * \param [in] mesh   The mesh.
+ * \param [in] qid    Process-local id of the quadrant (cumulative over trees).
+ * \return            A pointer to the requested quadrant.
+ */
+p4est_quadrant_t   *p4est_mesh_get_quadrant (p4est_t * p4est,
+                                             p4est_mesh_t * mesh,
+                                             p4est_locidx_t qid);
+
+/** Lookup neighboring quads of quadrant in a specific direction.
+ * \param [in]  p4est              Forest to be worked with.
+ * \param [in]  ghost              Ghost layer.
+ * \param [in]  mesh               Mesh structure.
+ * \param [in]  curr_quad_id       Process-local id of current quad.
+ * \param [in]  direction          Direction i in which to look for adjacent
+ *                                 quadrants is encoded as follows:
+ *                                  0 .. 3 neighbor(-s) across face i,
+ *                                  4 .. 7 neighbor(-s) across corner i-4.
+ * TODO: Allow any combination of empty output arrays.
+ * \param [out] neighboring_quads  Array containing neighboring quad(-s).
+ *                                 Needs to be empty on input, size of
+ *                                 p4est_quadrant_t *.  May be NULL, then
+ *                                 \b neighboring_qids must not be NULL.
+ * \param [out] neighboring_qids   Array containing quadrant ids for neighboring
+ *                                 quadrants. May be NULL, then no neighboring
+ *                                 qids are collected.
+ *                                 If non-NULL the array needs to be empty and
+ *                                 will contain int.
+ * CAUTION: Note, that the encodings differ from the encodings saved in the
+ *          mesh.
+ * TODO: Encodings are the same as in p4est_mesh for all quadrants.
+ * TODO: Ghosts can be encoded by returning the quad_to_quad convention in qid.
+ *       For ghost quadrants, we add -300 to the values in p4est_mesh.
+ *       This means that values below -100 belong to ghosts, values above to locals.
+ *          Positive values are for local quadrants, negative values indicate
+ *          ghost quadrants.
+ *          Faces:     1 ..   8 => same size neighbor
+ *                                 (r * 4 + nf) + 1; nf = 0 .. 3 face index;
+ *                                 r = 0 .. 1 relative orientation
+ *                     9 ..  24 => double size neighbor
+ *                                 9 + h * 8 + r * 4 + nf; h = 0 .. 1 number
+ *                                 of the subface; r, nf as above
+ *                    25 ..  32 => half-size neighbors
+ *                                 25 + r * 4 + nf; r, nf as above
+ *          Corners:   1 ..   4 => size not encoded for corners
+ *                                 nc + 1; nc = 0 .. 3 corner index
+ * \param [out] neighboring_encs   Array containing encodings for neighboring
+ *                                 quads.
+ *                                 Needs to be empty, contains int.
+ *
+ */
+p4est_locidx_t      p4est_mesh_get_neighbors (p4est_t * p4est,
+                                              p4est_ghost_t * ghost,
+                                              p4est_mesh_t * mesh,
+                                              p4est_locidx_t curr_quad_id,
+                                              p4est_locidx_t direction,
+                                              sc_array_t * neighboring_quads,
+                                              sc_array_t * neighboring_encs,
+                                              sc_array_t * neighboring_qids);
+
 /** Find a quadrant based on its cumulative number in the local forest.
+ * If the quad_to_tree field of the mesh structure exists, this is O(1).
+ * Otherwise, we perform a binary search over the processor-local trees.
+ *
  * \param [in]  p4est           Forest to be worked with.
+ * \param [in]  mesh            A mesh derived from the forest.
  * \param [in]  cumulative_id   Cumulative index over all trees of quadrant.
+ *                              Must refer to a local (non-ghost) quadrant.
  * \param [in,out] which_tree   If not NULL, the input value can be -1
  *                              or an initial guess for the quadrant's tree
  *                              and output is the tree of returned quadrant.
@@ -163,12 +253,13 @@ void                p4est_mesh_destroy (p4est_mesh_t * mesh);
  * \return                      The identified quadrant.
  */
 p4est_quadrant_t   *p4est_mesh_quadrant_cumulative (p4est_t * p4est,
+                                                    p4est_mesh_t * mesh,
                                                     p4est_locidx_t
                                                     cumulative_id,
-                                                    p4est_topidx_t
-                                                    * which_tree,
-                                                    p4est_locidx_t
-                                                    * quadrant_id);
+                                                    p4est_topidx_t *
+                                                    which_tree,
+                                                    p4est_locidx_t *
+                                                    quadrant_id);
 
 /** Initialize a mesh neighbor iterator by quadrant index.
  * \param [out] mfn         A p4est_mesh_face_neighbor_t to be initialized.
@@ -193,8 +284,8 @@ void                p4est_mesh_face_neighbor_init (p4est_mesh_face_neighbor_t
                                                    p4est_ghost_t * ghost,
                                                    p4est_mesh_t * mesh,
                                                    p4est_topidx_t which_tree,
-                                                   p4est_quadrant_t
-                                                   * quadrant);
+                                                   p4est_quadrant_t *
+                                                   quadrant);
 
 /** Move the iterator forward to loop around neighbors of the quadrant.
  * \param [in,out] mfn      Internal status of the iterator.
