@@ -454,7 +454,7 @@ p4est_nodes_foreach (void **item, const void *u)
   const p4est_locidx_t *new_node_number =
     (const p4est_locidx_t *) internal_data->user_data;
 
-  *item = (void *) (long) new_node_number[(long) *item];
+  *item = (void *) (size_t) new_node_number[(size_t) * item];
 
   return 1;
 }
@@ -1263,9 +1263,9 @@ p4est_nodes_new (p4est_t * p4est, p4est_ghost_t * ghost)
 #ifdef P4EST_ENABLE_MPI
   /* Wait and close all send requests. */
   if (send_requests.elem_count > 0) {
-    mpiret = MPI_Waitall ((int) send_requests.elem_count,
-                          (MPI_Request *) send_requests.array,
-                          MPI_STATUSES_IGNORE);
+    mpiret = sc_MPI_Waitall ((int) send_requests.elem_count,
+                             (MPI_Request *) send_requests.array,
+                             MPI_STATUSES_IGNORE);
     SC_CHECK_MPI (mpiret);
   }
   nodes->shared_offsets = shared_offsets;
