@@ -44,16 +44,22 @@
 /* redefine macros */
 #define P4EST_ONDISK_FORMAT             P8EST_ONDISK_FORMAT
 #define P4EST_DIM                       P8EST_DIM
+#define P4EST_DIM_POW                   P8EST_DIM_POW
 #define P4EST_FACES                     P8EST_FACES
 #define P4EST_CHILDREN                  P8EST_CHILDREN
 #define P4EST_HALF                      P8EST_HALF
 #define P4EST_FTRANSFORM                P8EST_FTRANSFORM
 #define P4EST_INSUL                     P8EST_INSUL
+#define P4EST_ONLY_P8_LAND              P8EST_ONLY_P8_LAND
+#define P4EST_ONLY_P8_COMMA             P8EST_ONLY_P8_COMMA
 #define P4EST_STRING                    P8EST_STRING
 #define P4EST_MAXLEVEL                  P8EST_MAXLEVEL
 #define P4EST_QMAXLEVEL                 P8EST_QMAXLEVEL
+#define P4EST_OLD_MAXLEVEL              P8EST_OLD_MAXLEVEL
+#define P4EST_OLD_QMAXLEVEL             P8EST_OLD_QMAXLEVEL
 #define P4EST_ROOT_LEN                  P8EST_ROOT_LEN
 #define P4EST_QUADRANT_LEN              P8EST_QUADRANT_LEN
+#define P4EST_QUADRANT_MASK             P8EST_QUADRANT_MASK
 #define P4EST_LAST_OFFSET               P8EST_LAST_OFFSET
 #define P4EST_QUADRANT_INIT             P8EST_QUADRANT_INIT
 #define P4EST_LEAF_IS_FIRST_IN_TREE     P8EST_LEAF_IS_FIRST_IN_TREE
@@ -95,6 +101,7 @@
 #define p4est_ghost_exchange_t          p8est_ghost_exchange_t
 #define p4est_indep_t                   p8est_indep_t
 #define p4est_nodes_t                   p8est_nodes_t
+#define p4est_lid_t                     p8est_lid_t
 #define p4est_lnodes_t                  p8est_lnodes_t
 #define p4est_lnodes_code_t             p8est_lnodes_code_t
 #define p4est_lnodes_rank_t             p8est_lnodes_rank_t
@@ -160,6 +167,8 @@
 #define p4est_find_corner_transform     p8est_find_corner_transform
 #define p4est_corner_array_index        p8est_corner_array_index
 #define p4est_connectivity_reorder      p8est_connectivity_reorder
+#define p4est_connectivity_reorder_newid                \
+        p8est_connectivity_reorder_newid
 #define p4est_connectivity_permute      p8est_connectivity_permute
 #define p4est_connectivity_join_faces   p8est_connectivity_join_faces
 #define p4est_connectivity_is_equivalent p8est_connectivity_is_equivalent
@@ -179,6 +188,7 @@
 #define p4est_balance                   p8est_balance
 #define p4est_partition                 p8est_partition
 #define p4est_checksum                  p8est_checksum
+#define p4est_checksum_partition        p8est_checksum_partition
 #define p4est_save                      p8est_save
 #define p4est_load                      p8est_load
 #define p4est_connect_type_int          p8est_connect_type_int
@@ -191,6 +201,28 @@
 
 /* functions in p4est_extended */
 #define p4est_replace_t                 p8est_replace_t
+#define p4est_lid_compare               p8est_lid_compare
+#define p4est_lid_is_equal              p8est_lid_is_equal
+#define p4est_lid_init                  p8est_lid_init
+#define p4est_lid_set_zero              p8est_lid_set_zero
+#define p4est_lid_set_one               p8est_lid_set_one
+#define p4est_lid_set_uint64            p8est_lid_set_uint64
+#define p4est_lid_chk_bit               p8est_lid_chk_bit
+#define p4est_lid_set_bit               p8est_lid_set_bit
+#define p4est_lid_copy                  p8est_lid_copy
+#define p4est_lid_add                   p8est_lid_add
+#define p4est_lid_sub                   p8est_lid_sub
+#define p4est_lid_bitwise_neg           p8est_lid_bitwise_neg
+#define p4est_lid_bitwise_or            p8est_lid_bitwise_or
+#define p4est_lid_bitwise_and           p8est_lid_bitwise_and
+#define p4est_lid_shift_right           p8est_lid_shift_right
+#define p4est_lid_shift_left            p8est_lid_shift_left
+#define p4est_lid_add_inplace           p8est_lid_add_inplace
+#define p4est_lid_sub_inplace           p8est_lid_sub_inplace
+#define p4est_lid_bitwise_or_inplace    p8est_lid_bitwise_or_inplace
+#define p4est_lid_bitwise_and_inplace   p8est_lid_bitwise_and_inplace
+#define p4est_quadrant_linear_id_ext128 p8est_quadrant_linear_id_ext128
+#define p4est_quadrant_set_morton_ext128 p8est_quadrant_set_morton_ext128
 #define p4est_new_ext                   p8est_new_ext
 #define p4est_mesh_new_ext              p8est_mesh_new_ext
 #define p4est_copy_ext                  p8est_copy_ext
@@ -216,6 +248,7 @@
 #define p4est_new_points                p8est_new_points
 
 /* functions in p4est_bits */
+#define p4est_quadrant_pad              p8est_quadrant_pad
 #define p4est_quadrant_print            p8est_quadrant_print
 #define p4est_quadrant_is_equal         p8est_quadrant_is_equal
 #define p4est_quadrant_overlaps         p8est_quadrant_overlaps
@@ -285,10 +318,15 @@
 #define p4est_quadrant_shift_corner     p8est_quadrant_shift_corner
 #define p4est_quadrant_linear_id        p8est_quadrant_linear_id
 #define p4est_quadrant_set_morton       p8est_quadrant_set_morton
+#define p4est_quadrant_successor        p8est_quadrant_successor
+#define p4est_quadrant_predecessor      p8est_quadrant_predecessor
+#define p4est_quadrant_srand            p8est_quadrant_srand
 
 /* functions in p4est_search */
+#define p4est_find_partition            p8est_find_partition
 #define p4est_find_lower_bound          p8est_find_lower_bound
 #define p4est_find_higher_bound         p8est_find_higher_bound
+#define p4est_find_quadrant_cumulative  p8est_find_quadrant_cumulative
 #define p4est_split_array               p8est_split_array
 #define p4est_find_range_boundaries     p8est_find_range_boundaries
 #define p4est_search                    p8est_search
@@ -311,6 +349,7 @@
 #define p4est_tree_is_complete          p8est_tree_is_complete
 #define p4est_tree_print                p8est_tree_print
 #define p4est_is_equal                  p8est_is_equal
+#define p4est_quadrant_copy             p8est_quadrant_copy
 #define p4est_is_valid                  p8est_is_valid
 #define p4est_tree_compute_overlap      p8est_tree_compute_overlap
 #define p4est_tree_uniqify_overlap      p8est_tree_uniqify_overlap
@@ -345,12 +384,18 @@
 #define p4est_comm_neighborhood_owned   p8est_comm_neighborhood_owned
 #define p4est_comm_sync_flag            p8est_comm_sync_flag
 #define p4est_comm_checksum             p8est_comm_checksum
+#define p4est_comm_checksum_partition   p8est_comm_checksum_partition
 #define p4est_transfer_fixed            p8est_transfer_fixed
+#define p4est_bsearch_partition         p8est_bsearch_partition
 #define p4est_transfer_fixed_begin      p8est_transfer_fixed_begin
 #define p4est_transfer_fixed_end        p8est_transfer_fixed_end
 #define p4est_transfer_custom           p8est_transfer_custom
 #define p4est_transfer_custom_begin     p8est_transfer_custom_begin
 #define p4est_transfer_custom_end       p8est_transfer_custom_end
+#define p4est_transfer_items            p8est_transfer_items
+#define p4est_transfer_items_begin      p8est_transfer_items_begin
+#define p4est_transfer_items_end        p8est_transfer_items_end
+#define p4est_transfer_end              p8est_transfer_end
 
 /* functions in p4est_io */
 #define p4est_deflate_quadrants         p8est_deflate_quadrants
@@ -369,6 +414,7 @@
 #define p4est_vtk_write_file            p8est_vtk_write_file
 #define p4est_vtk_write_header          p8est_vtk_write_header
 #define p4est_vtk_write_cell_dataf      p8est_vtk_write_cell_dataf
+#define p4est_vtk_write_cell_datav      p8est_vtk_write_cell_datav
 #define p4est_vtk_write_cell_data       p8est_vtk_write_cell_data
 #define p4est_vtk_write_point_dataf     p8est_vtk_write_point_dataf
 #define p4est_vtk_write_point_data      p8est_vtk_write_point_data
@@ -427,6 +473,8 @@
 #define p4est_mesh_memory_used          p8est_mesh_memory_used
 #define p4est_mesh_new                  p8est_mesh_new
 #define p4est_mesh_destroy              p8est_mesh_destroy
+#define p4est_mesh_get_quadrant         p8est_mesh_get_quadrant
+#define p4est_mesh_get_neighbors        p8est_mesh_get_neighbors
 #define p4est_mesh_quadrant_cumulative  p8est_mesh_quadrant_cumulative
 #define p4est_mesh_face_neighbor_init   p8est_mesh_face_neighbor_init
 #define p4est_mesh_face_neighbor_init2  p8est_mesh_face_neighbor_init2
