@@ -55,6 +55,15 @@ SC_EXTERN_C_BEGIN;
 /** The size of insulation layer */
 #define P8EST_INSUL 27
 
+/** Only use logical AND term in 3D */
+#define P8EST_ONLY_P8_LAND(x) && (x)
+
+/** Only use comma and expression in 3D */
+#define P8EST_ONLY_P8_COMMA(x) , (x)
+
+/** Exponentiate with dimension */
+#define P8EST_DIM_POW(a) ((a) * (a) * (a))
+
 /* size of face transformation encoding */
 #define P8EST_FTRANSFORM 9
 
@@ -520,7 +529,7 @@ p8est_connectivity_t *p8est_connectivity_inflate (sc_array_t * buffer);
  * \return              Returns valid connectivity, or NULL on file error.
  */
 p8est_connectivity_t *p8est_connectivity_load (const char *filename,
-                                               size_t * bytes);
+                                               size_t *bytes);
 
 /** Create a connectivity structure for the unit cube.
  */
@@ -582,6 +591,22 @@ p8est_connectivity_t *p8est_connectivity_new_shell (void);
  */
 p8est_connectivity_t *p8est_connectivity_new_sphere (void);
 
+/** Create a connectivity structure that builds a revolution torus.
+ *
+ * This connectivity reuses vertices and relies on a geometry transformation.
+ * It is thus not suitable for p8est_connectivity_complete.
+ *
+ * This connectivity reuses ideas from disk2d connectivity.
+ * More precisely the torus is divided into segments arround
+ * the revolution axis, each segments is made of 5 trees (à la disk2d).
+ * The total number of trees if 5 times the number of segments.
+ *
+ * This connectivity is meant to be used with \ref p8est_geometry_new_torus
+ *
+ * \param[in] nSegments number of trees along the great circle
+ */
+p8est_connectivity_t *p8est_connectivity_new_torus (int nSegments);
+
 /** Create connectivity structure from predefined catalogue.
  * \param [in]  name            Invokes connectivity_new_* function.
  *              brick235        brick (2, 3, 5, 0, 0, 0)
@@ -601,8 +626,9 @@ p8est_connectivity_t *p8est_connectivity_new_byname (const char *name);
  * This is useful if you would like to uniformly refine by something other
  * than a power of 2.
  *
- * \param [in] conn         a valid connectivity
- * \param [in] num_per_edge the number of new trees in each direction
+ * \param [in] conn         A valid connectivity
+ * \param [in] num_per_edge The number of new trees in each direction.
+ *                      Must use no more than \ref P8EST_OLD_QMAXLEVEL bits.
  *
  * \return a refined connectivity.
  */
@@ -704,7 +730,7 @@ void                p8est_connectivity_permute (p8est_connectivity_t * conn,
 
 #ifdef P4EST_WITH_METIS
 
-/** p8est_connectivity_reorder
+/** Reorder a connectivity using METIS.
  * This function takes a connectivity \a conn and a parameter \a k,
  * which will typically be the number of processes, and reorders the trees
  * such that if every processes is assigned (num_trees / k) trees, the
@@ -730,6 +756,33 @@ void                p8est_connectivity_reorder (sc_MPI_Comm comm, int k,
                                                 p8est_connectivity_t * conn,
                                                 p8est_connect_type_t ctype);
 
+/** Reorder a connectivity using METIS.
+ * This is the same form of p8est_connectivity_reorder but it takes an initialized
+ * sc array \a newid as extra argument.
+ * In this way, the users can map old indices to new indices in the case it
+ * is necessary (for instance to retrieve high-order nodes previously stored
+ * in an array with old indices).
+ * \param [in]     comm       MPI communicator.
+ * \param [in]     k          if k > 0, the number of pieces metis will use to
+ *                            guide the reordering; if k = 0, the number of
+ *                            pieces will be determined from the MPI
+ *                            communicator.
+ * \param [in,out] conn       connectivity that will be reordered.
+ * \param [in]     ctype      determines when an edge exists in the dual graph
+ *                            of the connectivity structure.
+ * \param [in,out] newid      array that maps old tree indices to new ones.
+ *                            newid has to be an sc_array and it has to be
+ *                            initialized (non-NULL) with element size
+ *                            of size_t (using sc_array_new (sizeof (size_t))).
+ *                            Input length arbitrary, output length modified.
+ */
+sc_array_t         *p8est_connectivity_reorder_newid (sc_MPI_Comm comm, int k,
+                                                      p8est_connectivity_t *
+                                                      conn,
+                                                      p8est_connect_type_t
+                                                      ctype,
+                                                      sc_array_t * newid);
+
 #endif /* P4EST_WITH_METIS */
 
 /** p8est_connectivity_join_faces
