@@ -34,9 +34,11 @@
 
 typedef enum
 {
-  P8EST_GEOMETRY_BUILTIN_MAGIC = 0x65F2F8DF,
+  P8EST_GEOMETRY_BUILTIN_MAGIC = 0x30F3F8DF,
   P8EST_GEOMETRY_BUILTIN_SHELL,
-  P8EST_GEOMETRY_BUILTIN_SPHERE
+  P8EST_GEOMETRY_BUILTIN_SPHERE,
+  P8EST_GEOMETRY_BUILTIN_TORUS,
+  P8EST_GEOMETRY_LAST
 }
 p8est_geometry_builtin_type_t;
 
@@ -58,6 +60,16 @@ typedef struct p8est_geometry_builtin_sphere
 }
 p8est_geometry_builtin_sphere_t;
 
+typedef struct p8est_geometry_builtin_torus
+{
+  p8est_geometry_builtin_type_t type;
+  int                 nSegments;
+  double              R2, R1, R0;
+  double              R1byR0, R0sqrbyR1, R0log;
+  double              Clength;
+}
+p8est_geometry_builtin_torus_t;
+
 typedef struct p8est_geometry_builtin
 {
   /** The geom member needs to come first; we cast to p8est_geometry_t * */
@@ -67,6 +79,7 @@ typedef struct p8est_geometry_builtin
     p8est_geometry_builtin_type_t type;
     p8est_geometry_builtin_shell_t shell;
     p8est_geometry_builtin_sphere_t sphere;
+    p8est_geometry_builtin_torus_t torus;
   }
   p;
 }
@@ -299,3 +312,147 @@ p8est_geometry_new_sphere (p8est_connectivity_t * conn,
 
   return (p8est_geometry_t *) builtin;
 }
+
+/**
+ * geometric coordinate transformation for torus of revolution geometry.
+ *
+ * Define the geometric transformation from logical space (where AMR
+ * is performed) to the physical space.
+ *
+ * \param[in]  p4est      the forest
+ * \param[in]  which_tree tree id inside forest
+ * \param[in]  rst        coordinates in AMR space : [0,1]^3
+ * \param[out] xyz        cartesian coordinates in physical space after geometry
+ *
+ * Note abc[3] contains cartesian coordinates in logical
+ * vertex space (before geometry).
+ */
+static void
+p8est_geometry_torus_X (p8est_geometry_t * geom,
+                        p4est_topidx_t which_tree,
+                        const double rst[3], double xyz[3])
+{
+  const struct p8est_geometry_builtin_torus *torus
+    = &((p8est_geometry_builtin_t *) geom)->p.torus;
+  double              x, y, R, q;
+  double              abc[3];
+
+  (void) y;
+
+  /* transform from the reference cube [0,1]^3 into logical vertex space
+     using bi/trilinear transformation */
+  p4est_geometry_connectivity_X (geom, which_tree, rst, abc);
+
+  /*
+   * assert that input points are in the expected range
+   * Note: maybe we should remove these assert, this would allow
+   * ghost quadrant at external boundary to call this routine ?
+   */
+
+  p4est_topidx_t      which_tree_local = which_tree % 5;
+
+  P4EST_ASSERT (torus->type == P8EST_GEOMETRY_BUILTIN_TORUS);
+  P4EST_ASSERT (abc[0] < 1.0 + SC_1000_EPS && abc[0] > -1.0 - SC_1000_EPS);
+  if (which_tree_local < 4)
+    P4EST_ASSERT (abc[1] < 2.0 + SC_1000_EPS && abc[1] > 1.0 - SC_1000_EPS);
+  else
+    P4EST_ASSERT (abc[1] < 1.0 + SC_1000_EPS && abc[1] > -1.0 - SC_1000_EPS);
+
+  /* abc[2] is always 0 here and so unused in 2D ... */
+
+  if (which_tree_local < 4) {
+    double              p, tanx;
+
+    p = 2.0 - abc[1];
+    tanx = -tan (abc[0] * M_PI_4);      /* x = tan (theta) */
+
+    x = p * (-abc[0]) + (1. - p) * tanx;
+
+    /* compute transformation ingredients */
+    R = torus->R0sqrbyR1 * pow (torus->R1byR0, abc[1]);
+
+    /* R*cos(theta) */
+    /* q = R / sqrt (x * x + 1.); */
+    q = R / sqrt (1. + (1. - p) * (tanx * tanx) + 1. * p);
+
+    /* assign correct coordinates based on patch id */
+
+    switch (which_tree_local) {
+    case 0:                    /* bottom */
+      xyz[0] = +q;              /*   R*cos(theta) */
+      xyz[1] = +q * x;          /*   R*sin(theta) */
+      break;
+    case 1:                    /* right */
+      xyz[0] = +q * x;          /*   R*sin(theta) = R*cos(theta-PI/2) */
+      xyz[1] = -q;              /* - R*cos(theta) = R*sin(theta-PI/2) */
+      break;
+    case 2:                    /* top */
+      xyz[0] = -q;              /* - R*cos(theta) = R*cos(theta-PI) */
+      xyz[1] = -q * x;          /* - R*sin(theta) = R*sin(theta-PI) */
+      break;
+    case 3:                    /* left */
+      xyz[0] = -q * x;          /* -R*sin(theta) = R*cos(theta-3*PI/2) */
+      xyz[1] = +q;              /*  R*cos(theta) = R*sin(theta-3*PI/2) */
+      break;
+    default:
+      SC_ABORT_NOT_REACHED ();
+    }
+
+  }
+  else {
+
+    /* center square */
+    xyz[0] = abc[0] * torus->Clength;
+    xyz[1] = abc[1] * torus->Clength;
+
+  }
+
+  /* translation */
+  xyz[0] += torus->R2;
+
+  /* rotate around Y-axis */
+  {
+
+    int                 iSegment = which_tree / 5;
+
+    double              tmp = xyz[0];
+    double              phi =
+      2 * M_PI / torus->nSegments * (iSegment + abc[2]);
+    xyz[0] = tmp * cos (phi);
+    xyz[2] = tmp * sin (phi);
+  }
+
+}                               /* p8est_geometry_torus_X */
+
+p8est_geometry_t   *
+p8est_geometry_new_torus (p8est_connectivity_t * conn,
+                          double R0, double R1, double R2)
+{
+  p8est_geometry_builtin_t *builtin;
+  struct p8est_geometry_builtin_torus *torus;
+
+  builtin = P4EST_ALLOC_ZERO (p8est_geometry_builtin_t, 1);
+
+  torus = &builtin->p.torus;
+  torus->type = P8EST_GEOMETRY_BUILTIN_TORUS;
+  torus->R0 = R0;
+  torus->R1 = R1;
+  torus->R2 = R2;
+  torus->nSegments = conn->num_trees / 5;
+
+  /* variables useful for the outer shell */
+  torus->R1byR0 = R1 / R0;
+  torus->R0sqrbyR1 = R0 * R0 / R1;
+  torus->R0log = log (R1 / R0);
+
+  /* variables useful for the center square */
+  torus->Clength = R0 / sqrt (2.);
+  /* torus->CdetJ = pow (R0 / sqrt (3.), 3.); */
+
+  builtin->geom.name = "p8est_torus";
+  builtin->geom.user = conn;
+  builtin->geom.X = p8est_geometry_torus_X;
+
+  return (p8est_geometry_t *) builtin;
+
+}                               /* p8est_geometry_new_torus */
